function [alpha, beta] = hmm_fwdbck(table, model, obs)
% HMM_FWDBCK	Implementation of Forward backward algorithm
%
%    Function returns a vector of Alpha and Beta factors
%
%    TABLE is a set of examples
%    MODEL is a structure describing HMM model
%    OBS is a set of observation probabilities
%      obs = get_obsprob(table, model);
%


% Compute the observation likelihoods
if nargin < 3
  obs = get_obsprob(table, model);
end
n = length(obs);

% Forward pass
transition = model.transition;
alpha(1) = multiply(model.prior, obs(1));
for i=2:n
  transition.vars=[i-1 i];
  factorA(i) = multiply(alpha(i-1), transition);
  factorA(i) = multiply(factorA(i), obs(i));
  %factorA(i).table = factorA(i).table ./ sum(sum(factorA(i).table));
  alpha(i) = maximum(factorA(i), i);
end

% Backward pass
beta(n) = table_factor(n, ones(26,1));
transition = model.transition;
for i=n-1:-1:1,
  transition.vars=[i i+1];
  factorB(i) = multiply(beta(i+1), transition);
  factorB(i) = multiply(factorB(i), obs(i+1));
  %factorB(i).table = factorB(i).table ./ sum(sum(factorB(i).table));
  beta(i) = maximum(factorB(i), i);
end

