function g = multiply(f1, f2)
% MULTIPLY     Multiplies two table factors
%
%    NEW_FACTOR = MULTIPLY(F1, F2) returns a factor over
%    union(F1.vars, F2. vars) s.t. for any assignment x over the
%    variables of NEW_FACTOR, NEW_FACTOR(x)=F1(x<F1.VARS>)*F2(x<F2.VARS>);
%

if 1
%assert(length(f1.vars)<=2 && length(f2.vars)<=2);

if length(f1.vars)>length(f2.vars)
  dummy=f1; f1=f2; f2=dummy;
end

if isempty(f1.vars)
  g.vars =f2.vars;
  g.table=f1.table*f2.table;
elseif isscalar(f1.vars)
  % 2 cases: a) f2 is a scalar b) f2 is a matrix
  if isscalar(f2.vars)
    if f1.vars==f2.vars
      g.vars=f1.vars;
      g.table=f1.table.*f2.table;
    else
      g.vars=[f1.vars f2.vars];
      g.table=f1.table*f2.table';
    end
  else
    g.vars = f2.vars;
    if f1.vars==f2.vars(1)
      g.table=(f1.table*ones(1,size(f2.table,2))).*f2.table;
    elseif f1.vars==f2.vars(2)
      g.table=(ones(size(f2.table,1),1)*(f1.table')).*f2.table;
    else
      error('Multiplication would create a factor with >2 vars.');
    end
  end
else
  if f1.vars==f2.vars
    g.vars=f1.vars;
    g.table=(f1.table).*(f2.table);
  elseif f1.vars([2 1])==f2.vars
    f.vars=f1.vars;
    g.table=(f1.table).*(f2.table');
  else
    error('Multiplication would create a factor with >2 vars.');
  end
end
  




else  

  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % Conceptually correct implementation
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  
  
if max(length(f1.vars),2)~=ndims(f1.table) || ...
   max(length(f2.vars),2)~=ndims(f2.table)
  error('Inconsistent factor structure');
end

f.vars=union(f1.vars, f2.vars);
n=length(f.vars);
dims=ones(1,max(n,2)); %min is b/c of Matlab ugliness with zeros/ones

if n==0
  %Unity factor
  warning('Unity factor');
  f.table=ones(1,1);
  return;
end

for i=1:n,
  j1=find(f1.vars==f.vars(i));
  j2=find(f2.vars==f.vars(i));
  
  if length(j1)==1 && length(j2)==1
    size1=size(f1.table, j1);
    size2=size(f2.table, j2);
    if size1==size2
      dims(i)=size1;
    else
      error('Inconsistent dimensions of variables.')
    end
  elseif length(j1)==1 && length(j2)==0
    dims(i)=size(f1.table, j1);
  elseif length(j2)==1 && length(j1)==0
    dims(i)=size(f2.table, j2);
  else
    error('Inconsistent indexing.')
  end
end

celldims=num2cell(dims);
f.table=zeros(celldims{:});

% Now compute the assignments
asg.vars=f.vars;
asg.vals=ones(1,n);
done=0;

while ~done,
  val1=factorVal(f1,asg);
  val2=factorVal(f2,asg);
  cellasg=num2cell(asg.vals);
  f.table(cellasg{:})=val1*val2;
  
  %advance to the next assignment
  [asg,done]=advanceAssignment(asg, dims);
end

g=f;

end
